/*
 * ServletControlador.java
 *
 * Created on 8 de mayo de 2007, 12:55
 */

package contweb.mvc;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

public class ServletControlador extends HttpServlet  {
    private static final String INICIALIZADOR = "inicializador"; //Debe coincidir con el DD.
    private static final String PREFIJO_EVENTO = "evento.";    //Debe coincidir con el DD.
    private static final String NOM_TABLA_EVENTOS = "tablaEventos";
  
    public void init(ServletConfig config) throws ServletException {
        super.init(config);

        try {
            //Recuperar la clase inicializadora de la aplicacin (ver DD).
            String inicializador = config.getInitParameter(INICIALIZADOR);
            Inicializador ini = (Inicializador)Class.forName(inicializador).newInstance();
            ini.init(config);
            //Recuperar las clases de los eventos
            Map eventos = new HashMap();
            Enumeration e = config.getInitParameterNames();

            //Recorrer los parmetros de inicio buscando eventos
            while (e.hasMoreElements()) {
                String nombre = (String)e.nextElement();
                if (nombre.startsWith(PREFIJO_EVENTO)) { //Es un evento
                    String clase = config.getInitParameter(nombre);   //Clase que maneja el evento
                    Evento evento = (Evento)Class.forName(clase).newInstance();
                    eventos.put(nombre, evento);
                    mostrar("Clase " + clase + " registrada para eventos de tipo " + nombre);
                }
            }

            //Guardar la tabla de eventos en el contexto
            config.getServletContext().setAttribute(NOM_TABLA_EVENTOS, eventos);
        } catch (Exception ex) {
            mostrar(ex.getMessage());
            ex.printStackTrace();
            throw new UnavailableException(ex.getMessage());
        }
    } 
    public void doPost(HttpServletRequest req, HttpServletResponse res) 
						throws ServletException, IOException {
        String nomEvento = req.getParameter(Evento.NOM_EVENTO);
        Map eventos = (Map)getServletContext().getAttribute(NOM_TABLA_EVENTOS);

        if (!eventos.containsKey(nomEvento)) {  //Si no se encuentra el evento se lanza excepcin
            String msg = "Evento no encontrado " + nomEvento;
            mostrar(msg);
        throw new ServletException(msg);
        } else {         //Si se encuentra el evento se procesa
            mostrar("Procesando evento " + nomEvento);
            Evento evento = (Evento)eventos.get(nomEvento);           //Se recupera la clase controladora
            String path = evento.procesar(getServletContext(), req);   //Se delega el evento del proceso
            mostrar("Evento procesado, redirigiendo a " + path);
            
            //Se redirige la peticin usando opcin 1: getServletContext 
            getServletContext().getRequestDispatcher(path).forward(req,res);
            
            //Se redirige la peticin usando opcin 2: objeto req
            //req.getRequestDispatcher(path).forward(req, res);           
            
            mostrar("Evento " + nomEvento + " procesado");
        }
    }

    private void mostrar(String msg) {
        System.out.println(msg);
        log(msg);
    }
}

